@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -377,8 +377,9 @@ void slrn_art_mark_signature (Slrn_Article_Type *a) /*{{{*/
    /* skip back until a line matches the signature RegExp */
    
    while ((l != NULL) && (0 == (l->flags & HEADER_LINE))
-	  && (NULL == is_matching_line ((unsigned char *) l->buf,
-					Slrn_Strip_Sig_Regexp)))
+	  && ((l->flags & VERBATIM_LINE) ||
+	      (NULL == is_matching_line ((unsigned char *) l->buf,
+					 Slrn_Strip_Sig_Regexp))))
      l = l->prev;
    
    if ((l == NULL) || (l->flags & HEADER_LINE)) return;
@@ -1030,7 +1031,7 @@ void slrn_art_mark_pgp_signature (Slrn_Article_Type *a) /*{{{*/
 	     l->flags &= ~QUOTE_LINE;
 	     
 	     if ((NULL != (l = l->next)) &&
-		 (*l->buf != 0))
+		 !strncmp (l->buf, "Hash: ", 6))
 	       {
 		  /* catch the `Hash: ... ' line */
 		  l->flags |= PGP_SIGNATURE_LINE;
@@ -1038,6 +1039,16 @@ void slrn_art_mark_pgp_signature (Slrn_Article_Type *a) /*{{{*/
 		  
 		  l = l->next;
 	       }
+	     if ((NULL != l) &&
+		 !strncmp (l->buf, "NotDashEscaped: ", 16))
+	       {
+		  /* the optional "NotDashEscaped: ..." line */
+		  l->flags |= PGP_SIGNATURE_LINE;
+		  l->flags &= ~QUOTE_LINE;
+		  
+		  l = l->next;		  
+	       }
+	     
 	     continue;
 	  }
 
@@ -1060,14 +1071,14 @@ void slrn_art_mark_pgp_signature (Slrn_Article_Type *a) /*{{{*/
 		  l = l->next;
 		  continue;
 	       }
-	     
+
 	     l0->flags |= PGP_SIGNATURE_LINE;
-	     l0->flags &= ~QUOTE_LINE;
+	     l0->flags &= ~(QUOTE_LINE|SIGNATURE_LINE);
 	     do
 	       {
 		  l0 = l0->next;
 		  l0->flags |= PGP_SIGNATURE_LINE;
-		  l0->flags &= ~QUOTE_LINE;
+		  l0->flags &= ~(QUOTE_LINE|SIGNATURE_LINE);
 	       }
 	     while (l0 != l);
 	     return;
