@@ -650,7 +650,7 @@ static void decode_mime_base64 (Slrn_Article_Type *a)
    l = a->lines;
    
    /* skip header and separator */
-   while (((l != NULL) && (l->flags & HEADER_LINE)) || l->buf[0] == '\0')
+   while ((l != NULL) && ((l->flags & HEADER_LINE) || l->buf[0] == '\0'))
      l = l->next;
    
    if (l == NULL) return;
@@ -1548,14 +1548,14 @@ Content-Transfer-Encoding: %s\n",
      }
 }
 
-FILE *slrn_mime_encode (FILE *fp)
+VFILE *slrn_mime_encode (VFILE *vp)
 {
    if ((Mime_Posting_Encoding == ENCODED_7BIT)
        || (Mime_Posting_Encoding == ENCODED_8BIT))
-     return fp;
+     return vp;
    
    /* Add encoding later. */
-   return fp;
+   return vp;
 }
 
 #endif /* NOT SLRNPULL_CODE */
