@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -124,6 +124,8 @@ char *Data_Dir = "data";
 static char *Active_File = "active";
 
 static int Stdout_Is_TTY;
+static int Fetch_Score;
+static int Use_Fetch_Score;
 static int Kill_Score;
 static char *Active_Groups_File;
 static time_t Start_Time;
@@ -182,11 +184,12 @@ typedef struct _Active_Group_Type /*{{{*/
 /*}}}*/
 Active_Group_Type;
 
-static char *Current_Newsgroup;
+static Active_Group_Type *Current_Group;
 
 static Active_Group_Type *Active_Groups;
 static Active_Group_Type *Active_Groups_Tail;
 
+static int write_headers_file (Active_Group_Type *group);
 /*}}}*/
 
 static FILE *MLog_Fp;
@@ -1073,7 +1076,7 @@ static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
    if (do_log)
      {
 	log_message (_("%s: %u/%u (%u killed), Time: %02u:%02u:%02u, BPS: %lu"),
-		     Current_Newsgroup, 
+		     Current_Group->name, 
 		     Num_Articles_Received, Num_Articles_To_Receive, Num_Killed,
 		     hour, min, sec,
 		     in / elapsed_time);
@@ -1097,7 +1100,7 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
 	   && (EOF == fprintf (fp, "\tXref: %s", xov->xref)))
        || (EOF == fputc ('\n', fp)))
      {
-	log_error (_("Error writing to overview database: %s:%d."), Current_Newsgroup, xov->id);
+	log_error (_("Error writing to overview database: %s:%d."), Current_Group->name, xov->id);
 	return -1;
      }
 #else
@@ -1281,7 +1284,7 @@ static int get_bodies (NNTP_Type *s, int *numbers, /*{{{*/
 
 /*}}}*/
 
-static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov) /*{{{*/
+static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **headers, Slrn_XOver_Type *xov) /*{{{*/
 {
    int status;
    Slrn_Header_Type h;
@@ -1323,17 +1326,19 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
 #endif
    
 #if 1
-   (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
+   (void) is_msgid_cached (h.msgid, Current_Group->name, (unsigned int) n, 1);
 #endif
 
-   score = slrn_score_header (&h, Current_Newsgroup, (KLog_Fp != NULL) ? &sdi : NULL);
-   if (score < Kill_Score)
+   score = slrn_score_header (&h, Current_Group->name, (KLog_Fp != NULL) ? &sdi : NULL);
+   if ((score < Kill_Score) || ((score >= Fetch_Score) && Use_Fetch_Score && g->headers_only))
      {
-	Num_Killed++;
 	if (KLog_Fp != NULL)
 	  {
 	     Slrn_Score_Debug_Info_Type *hlp = sdi;
-	     fprintf (KLog_Fp, _("Score %d killed article %s\n"), score, h.msgid);
+	     if (score < Kill_Score)
+	       fprintf (KLog_Fp, _("Score %d killed article %s\n"), score, h.msgid);
+	     else
+	       fprintf (KLog_Fp, _("Score %d requested article %s\n"), score, h.msgid);
 	     while (hlp != NULL)
 	       {
 		  if (hlp->description [0] != 0)
@@ -1347,10 +1352,16 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
 		  hlp = hlp->next;
 	       }
 	     fprintf (KLog_Fp, _("  Newsgroup: %s\n  From: %s\n  Subject: %s\n\n"),
-		      Current_Newsgroup, h.from, h.subject);
+		      Current_Group->name, h.from, h.subject);
 	  }
-	slrn_free (*headers);
-	*headers = NULL;
+	if (score < Kill_Score)
+	  {
+	     Num_Killed++;
+	     slrn_free (*headers);
+	     *headers = NULL;
+	  }
+	else
+	  g->requests = slrn_ranges_add (g->requests, n, n);
 /*	return 0; */
      }
    while (sdi != NULL)
@@ -1367,14 +1378,14 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
 #endif
 #if 0
    /* This next call should add the message id to the cache. */
-   (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
+   (void) is_msgid_cached (h.msgid, Current_Group->name, (unsigned int) n, 1);
 #endif
    return 0;
 }
 
 /*}}}*/
 
-static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
+static int get_heads (NNTP_Type *s,  Active_Group_Type *g, int *numbers, char **heads, /*{{{*/
 		      Slrn_XOver_Type *xovs, unsigned int num)
 {
    unsigned int i;
@@ -1404,7 +1415,7 @@ static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
 
    for (i = 0; i < num; i++)
      {
-	if (-1 == fetch_head (s, numbers[i], heads + i, xovs + i))
+	if (-1 == fetch_head (s, g, numbers[i], heads + i, xovs + i))
 	  return -1;
      }
    
@@ -1546,7 +1557,7 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
    int ret;
    FILE *fp;
    
-   if (-1 == get_heads (s, numbers, heads, xovs, num))
+   if (-1 == get_heads (s, g, numbers, heads, xovs, num))
      return -1;
 
    ret = 0;
@@ -1619,93 +1630,98 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, 
 			       int server_min, int server_max, int marked_bodies) /*{{{*/
 {
-   unsigned int gmin, gmax;
-   int *numbers;
-   unsigned int num_numbers, i, imin;
+   if (!marked_bodies)
+     {
+	unsigned int gmin, gmax;
+	int *numbers;
+	unsigned int num_numbers, i, imin;
+	
+	Num_Articles_Received = 0;
+	Num_Killed = 0;
+	Num_Articles_To_Receive = 0;
+	
+	gmin = g->min;
+	gmax = g->max;
+	
+	if (((server_min > server_max) || (server_max < 0))
+	    || (((unsigned int)server_max <= gmax) && (gmin <= gmax)))
+	  {
+	     log_message (_("%s: no new articles available."), g->name);
+	     return 0;
+	  }
+	
+	Num_Duplicates = 0;
+	numbers = list_server_numbers (s, g, server_min, server_max, &num_numbers);
+	if (Num_Duplicates)
+	  log_message (_("%u duplicates removed leaving %u/%u."), 
+		       Num_Duplicates, num_numbers, num_numbers + Num_Duplicates);
+	
+	if (numbers == NULL) return -1;
+	
+	i = 0;
+	while ((i < num_numbers) && (numbers[i] <= (int) gmax))
+	  i++;
+	
+	if (i == num_numbers)
+	  {
+	     g->max = g->server_max;
+	     log_message (_("%s: No new articles available."), g->name);
+	     slrn_free ((char *) numbers);
+	     return 0;
+	  }
+	
+	log_message (_("%s: %u articles available."), g->name, num_numbers - i);
+	
+	/* Hmmm...  How shall g->max_to_get be defined?  Here I assume that it 
+	 * means to attempt to retrieve the last max_to_get articles.
+	 */
+	if ((g->max_to_get != 0) && (g->max_to_get + i < num_numbers))
+	  {
+	     log_message (_("%s: Only retrieving last %u articles."), g->name, g->max_to_get);
+	     i = num_numbers - g->max_to_get;
+	  }
+	
+	imin = i;
+	
+	(void) slrn_open_score (g->name);
+	
+	Num_Articles_To_Receive = num_numbers - i;
+	
+	while (i < num_numbers)
+	  {
+	     int ns[SLRN_MAX_QUEUED];
+	     unsigned int j;
+	     
+	     j = 0;
+	     while ((i < num_numbers) && (j < SLRN_MAX_QUEUED))
+	       {
+		  ns[j] = numbers[i];
+		  i++;
+		  j++;
+	       }
+	     
+	     print_time_stats (s, 0);
+	     (void) get_articles (s, g, ns, j);
+	     
+	     Num_Articles_Received += j;
+	  }
+	
+	g->max = g->server_max;
+	
+	(void) slrn_close_score ();
+	slrn_clear_requested_headers ();
+	slrn_free ((char *) numbers);
+	
+	print_time_stats (s, 1);	
+     }
+
+   /* Now, fetch marked article bodies. */
    
    /* Don't request bodies that are no longer there. */
    if (server_min > 1)
      g->requests = slrn_ranges_remove (g->requests, 1, server_min-1);
    get_marked_bodies (s, g);
    
-   if (marked_bodies)
-     return 0;
-
-   Num_Articles_Received = 0;
-   Num_Killed = 0;
-   Num_Articles_To_Receive = 0;
-   
-   gmin = g->min;
-   gmax = g->max;
-   
-   if (((server_min > server_max) || (server_max < 0))
-       || (((unsigned int)server_max <= gmax) && (gmin <= gmax)))
-     {
-	log_message (_("%s: no new articles available."), g->name);
-	return 0;
-     }
-
-   Num_Duplicates = 0;
-   numbers = list_server_numbers (s, g, server_min, server_max, &num_numbers);
-   if (Num_Duplicates)
-     log_message (_("%u duplicates removed leaving %u/%u."), 
-		  Num_Duplicates, num_numbers, num_numbers + Num_Duplicates);
-	
-   if (numbers == NULL) return -1;
-   
-   i = 0;
-   while ((i < num_numbers) && (numbers[i] <= (int) gmax))
-     i++;
-   
-   if (i == num_numbers)
-     {
-	g->max = g->server_max;
-	log_message (_("%s: No new articles available."), g->name);
-	slrn_free ((char *) numbers);
-	return 0;
-     }
-   
-   log_message (_("%s: %u articles available."), g->name, num_numbers - i);
-   
-   /* Hmmm...  How shall g->max_to_get be defined?  Here I assume that it 
-    * means to attempt to retrieve the last max_to_get articles.
-    */
-   if ((g->max_to_get != 0) && (g->max_to_get + i < num_numbers))
-     {
-	log_message (_("%s: Only retrieving last %u articles."), g->name, g->max_to_get);
-	i = num_numbers - g->max_to_get;
-     }
-	
-   imin = i;
-
-   (void) slrn_open_score (g->name);
-   
-   Num_Articles_To_Receive = num_numbers - i;
-   
-   while (i < num_numbers)
-     {
-	int ns[SLRN_MAX_QUEUED];
-	unsigned int j;
-	
-	j = 0;
-	while ((i < num_numbers) && (j < SLRN_MAX_QUEUED))
-	  {
-	     ns[j] = numbers[i];
-	     i++;
-	     j++;
-	  }
-	
-	print_time_stats (s, 0);
-	(void) get_articles (s, g, ns, j);
-	
-	Num_Articles_Received += j;
-     }
-   
-   g->max = g->server_max;
-   
-   (void) slrn_close_score ();
-   slrn_clear_requested_headers ();
-   slrn_free ((char *) numbers);
    return 0;
 }
 
@@ -1740,12 +1756,12 @@ static int pull_news (NNTP_Type *s, int marked_bodies) /*{{{*/
 	if (g->server_max > (unsigned int) max)
 	  g->server_max = max;
 
-	Current_Newsgroup = g->name;
+	Current_Group = g;
 	
 	(void) get_group_articles (s, g, min, max, marked_bodies);
 	
-	print_time_stats (s, 1);
-
+	(void) write_headers_file (g);
+	
 	g = g->next;
      }	
 	     
@@ -1877,11 +1893,10 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
    n = 0;
    while (NULL != (df = slrn_read_dir (dp)))
      {
-	char *name;
-	
-	name = df->name;
+	char *name = df->name;
 
-	if (*name != 'X')
+	/* Don't post incomplete / backup files */
+	if ((*name != 'X') || (name[strlen(name)-1] == '~'))
 	  continue;
 	
 	if (-1 == slrn_dircat (Outgoing_Dir, name, file, sizeof (file)))
@@ -2099,6 +2114,8 @@ static void usage (char *pgm) /*{{{*/
   -h HOSTNAME          Hostname of NNTP server to connect to.\n\
   --debug FILE         Write dialogue with server to FILE.\n\
   --expire             Perform expiration, but do not pull news.\n\
+  --fetch-score SCORE  Fetch article bodies with a score of at least SCORE\n\
+                       (for use in \"true offline\" mode)\n\
   --help               Print this usage information.\n\
   --kill-log FILE      Keep a log of all killed articles in FILE.\n\
   --kill-score SCORE   Kill articles with a score below SCORE.\n\
@@ -2145,7 +2162,6 @@ static int do_expire (int rebuild);
 static int get_new_groups (NNTP_Type *s);
 static int read_authinfo (void);
 static int read_headers_files (void);
-static int write_headers_files (void);
 static void read_requests_files (void);
 
 int main (int argc, char **argv) /*{{{*/
@@ -2229,6 +2245,12 @@ int main (int argc, char **argv) /*{{{*/
 	     Kill_Score = atoi (*argv);
 	     argv++; argc--;
 	  }
+	else if (!strcmp (arg, "--fetch-score") && (argc > 0))
+	  {
+	     Fetch_Score = atoi (*argv);
+	     Use_Fetch_Score = 1;
+	     argv++; argc--;
+	  }
 	else if (!strcmp (arg, "--debug") && (argc > 0))
 	  {
 	     if (NULL == (Slrn_Debug_Fp = fopen (*argv, "w")))
@@ -2288,11 +2310,6 @@ int main (int argc, char **argv) /*{{{*/
      {
 	if (-1 == do_expire (expire_mode == 2))
 	  slrn_exit_error (NULL);
-	if (-1 == write_headers_files ())
-	  {
-	     Exit_Code = SLRN_EXIT_FILEIO;
-	     slrn_exit_error (NULL);
-	  }
 	
 	close_log_files ();
 	return 0;
@@ -2320,8 +2337,7 @@ int main (int argc, char **argv) /*{{{*/
      {
 	init_signals ();
 	pull_news (Pull_Server, marked_bodies);
-	if ((-1 == write_headers_files ()) | /* avoid short-circuit */
-	    (-1 == write_active ()))
+	if (-1 == write_active ())
 	  {
 	     Exit_Code = SLRN_EXIT_FILEIO;
 	     slrn_exit_error (NULL);
@@ -2544,6 +2560,8 @@ static int handle_interrupts (void) /*{{{*/
      {
 	log_error (_("Performing shutdown."));
 	write_active ();
+	if (Current_Group != NULL)
+	  write_headers_file (Current_Group);
 	Exit_Code = SLRN_EXIT_SIGNALED;
 	slrn_exit_error (_("Slrnpull exiting on signal."));
      }
@@ -2620,7 +2638,7 @@ static char *read_header_from_file (char *file, int *has_body)
      {
 	unsigned int len;
 	
-	if (*line == '\n')
+	if ((*line == '\n') || (*line == '\r'))
 	  break;
 
 	len = strlen (line);
@@ -2645,7 +2663,7 @@ static char *read_header_from_file (char *file, int *has_body)
 	buffer_len += len;
      }
    
-   *has_body = (line != NULL) && (*line == '\n');
+   *has_body = (line != NULL) && ((*line == '\n') || (*line == '\r'));
    
    fclose (fp);
    return mbuf;
@@ -3154,6 +3172,7 @@ static int do_expire (int rebuild) /*{{{*/
 	  }
 	
 	(void) expire_group (g, rebuild);
+	(void) write_headers_file (g);
 	g = g->next;
      }
    
@@ -3264,69 +3283,44 @@ static int read_headers_files (void)
    return 0;
 }
 
-static int write_headers_files (void)
+static int write_headers_file (Active_Group_Type *group)
 {
-   char backup_file[SLRN_MAX_PATH_LEN + 1];
    char head_file[SLRN_MAX_PATH_LEN + 1];
    FILE *fp;
    int have_backup = 0;
-   Active_Group_Type *group = Active_Groups;
-   int error = 0;
 
-   while (group != NULL)
+   if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, group->dirname,
+			   head_file, sizeof (head_file)))
+       || (-1 == slrn_dircat (head_file, Headers_File,
+			      head_file, sizeof (head_file))))
      {
-	if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, group->dirname,
-				head_file, sizeof (head_file)))
-	    || (-1 == slrn_dircat (head_file, Headers_File,
-				   head_file, sizeof (head_file))))
-	  {
-	     log_error (_("Unable to write headers file for group %s.\n"), group->name);
-	     error = -1;
-	     goto next_group;
-	  }
-	   
-#ifdef VMS
-	slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak",
-		       head_file);
-#else
-# ifdef SLRN_USE_OS2_FAT
-	slrn_os2_make_fat (backup_file, sizeof (backup_file), head_file,
-			   ".bak");
-# else
-	slrn_snprintf (backup_file, sizeof (backup_file), "%s~", head_file);
-# endif
-#endif
-
-	/* Save backup file in case anything goes wrong */
-	have_backup = 1;
-	if (-1 == rename (head_file, backup_file))
-	  have_backup = 0;
-
-	if (group->headers != NULL)
-	  {
-	     if (NULL == (fp = fopen (head_file, "w")))
-	       {
-		  if (have_backup) (void) rename (backup_file, head_file);
-		  log_error (_("Unable to write headers file for group %s.\n"), group->name);
-		  error = -1;
-		  goto next_group;
-	       }
-	     if ((-1 == slrn_ranges_to_newsrc_file (group->headers,0,fp)) |
-		 (-1 == slrn_fclose (fp)))
-	       {
-		  if (have_backup) (void) rename (backup_file, head_file);
-		  log_error (_("Error while writing headers file for group %s.\n"), group->name);
-		  error = -1;
-	       }
-	  }
-	  
-	if (have_backup) slrn_delete_file (backup_file);
-	
-	next_group:
-	group = group->next;
+	log_error (_("Unable to write headers file for group %s.\n"), group->name);
+	return -1;
      }
-
-   return error;
+   
+   /* Save backup file in case anything goes wrong */
+   have_backup = (0 == slrn_create_backup (head_file));
+   
+   if (group->headers != NULL)
+     {
+	if (NULL == (fp = fopen (head_file, "w")))
+	  {
+	     if (have_backup) slrn_restore_backup (head_file);
+	     log_error (_("Unable to write headers file for group %s.\n"), group->name);
+	     return -1;
+	  }
+	if ((-1 == slrn_ranges_to_newsrc_file (group->headers,0,fp)) | /* avoid short-circuit */
+	    (-1 == slrn_fclose (fp)))
+	  {
+	     if (have_backup) slrn_restore_backup (head_file);
+	     log_error (_("Error while writing headers file for group %s.\n"), group->name);
+	     return -1;
+	  }
+     }
+   
+   if (have_backup) slrn_delete_backup (head_file);
+   
+   return 0;
 }
 
 static void read_requests_files (void)
